local addon = CreateFrame("Frame", "dMinimap", Minimap)

-- hide those nasty buttons
local frames = {
	MinimapZoomIn,
	MinimapZoomOut,
	MinimapToggleButton,
	MinimapBorderTop,
	MiniMapWorldMapButton,
	MinimapBorder,
	MinimapZoneText,
}

---------------------------- CONFIGURATION ------------------------------
local anchorpoint = 'TOPRIGHT' -- default TOPLEFT
local parent = UIParent
local relativepoint = 'TOPRIGHT' -- default TOPLEFT
local xpos = -20 -- default 36
local ypos = -20 -- default -36
local scale = 0.90
local alpha = 0.8
local minimapshape = "square" -- square or round shape of minimap
local classcolors = false-- true = classcolors, false = custom color (see the line below)
local bliptex = 1  -- 1 = changes the track (blip) texture to a new one, 0 = uses Blizzards default textures
-- NOTE: If you change bliptex from 1 to 0 you have to relog to make the change happen, /consoloe reloadui or /rl won't be enough

-- true = show glow "overlay", false = hide glow "overlay"
local roundglow = true 
local squareglow = false	
	
-- change round border texture here (default = round5) -- round, round2, round3, round4, round5, round6	
local roundtex = "Interface\\AddOns\\shMinimap\\texture\\round3.tga"
-- change square border texture here (default = square5) -- square, square2, square3, square4, square5, square6	
local squaretex = "Interface\\AddOns\\shMinimap\\texture\\square2.tga"

------------------------- bordercolorz --------------------------- 
local bordercolorz = {0.35,0.3,0.3} 

---------------------------  position of PvP tracker --------------------------- 
local PosWTracker = false -- true = enable positioning of the PvP tracker, false = disable positioning of the PvP tracker
-- NOTE: If you change PosWTracker from true to false you have to relog to make the change happen, /consoloe reloadui or /rl won't be enough

local Wpos = "TOP"
local Wpos_x = 0
local Wpos_y = -50
---------------------------- END OF CONFIGURATION ------------------------------

---------------------------- PvP tracker ------------------------------	
if PosWTracker == true then
local PvPWatch = WorldStateAlwaysUpFrame
PvPWatch:ClearAllPoints()
PvPWatch:SetPoint(Wpos, UIParent, Wpos_x, Wpos_y)
PvPWatch.SetPoint = function() end		
end	

---------------------------- textures and stuff ------------------------------
local r, g, b = NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b
local event = function(self)

	Minimap:EnableMouseWheel(true)
	Minimap:SetScript("OnMouseWheel", function(self, z)
		local c = Minimap:GetZoom()
		if(z > 0 and c < 5) then
			Minimap:SetZoom(c + 1)
		elseif(z < 0 and c > 0) then
			Minimap:SetZoom(c - 1)
		end
	end)
	
Minimap:SetParent(UIParent)
MinimapCluster:EnableMouse(false)
	
---------------------------- square shape ------------------------------	
	if minimapshape == "square" then 
		function GetMinimapShape() return "SQUARE" end
		Minimap:SetMaskTexture"Interface\\AddOns\\shMinimap\\texture\\Mask.blp"

		local ssf = CreateFrame("Frame")
		ssf:SetParent(Minimap)
		ssf:ClearAllPoints()
		ssf:SetPoint("CENTER",Minimap,0,0)
		ssf:SetWidth(172)		
		ssf:SetHeight(172)
		ssf:SetFrameLevel(2)
		
		local squareshape = ssf:CreateTexture(nil,"BACKGROUND")
		squareshape:SetTexture(squaretex)
		squareshape:SetAllPoints(ssf)
		squareshape:SetAlpha(0.9)
				
		if classcolors == true then
			local r, g, b
			local class = select(2, UnitClass('player'))
			if class then
				r = RAID_CLASS_COLORS[class].r
				g = RAID_CLASS_COLORS[class].g
				b = RAID_CLASS_COLORS[class].b
			end
			squareshape:SetVertexColor(r, g, b)
			squareshape:SetAlpha(alpha)
			elseif classcolors == false then
			squareshape:SetVertexColor(unpack(bordercolorz))
			squareshape:SetAlpha(alpha)			
		end
			
		MiniMapBattlefieldBorder:Hide()
		MiniMapBattlefieldFrame:SetParent(Minimap)
		MiniMapBattlefieldFrame:ClearAllPoints()
		MiniMapBattlefieldFrame:SetPoint("TOPRIGHT", 0, -3)

		local mmmf = MiniMapMailFrame
		mmmf:SetParent(Minimap)
		mmmf:ClearAllPoints()
		mmmf:SetPoint("BOTTOMRIGHT",Minimap, 13, -16)
		mmmf:SetScale(1.6)
		
		MiniMapLFGFrame:ClearAllPoints()
		MiniMapLFGFrame:SetParent(Minimap)
		MiniMapLFGFrameBorder:SetAlpha(0)		
		MiniMapLFGFrame:SetPoint("BOTTOMLEFT", Minimap, -13, -12)
		MiniMapLFGFrame:SetScale(1.2)
		
		MiniMapInstanceDifficulty:ClearAllPoints()
		MiniMapInstanceDifficulty:SetPoint("TOPRIGHT", Minimap, 18, 9)
		MiniMapInstanceDifficulty:SetFrameStrata("HIGH")
		
		MiniMapMailBorder:Hide()
		MiniMapMailIcon:SetTexture"Interface\\AddOns\\shMinimap\\texture\\mail"

		if squareglow == true then
			local fmi = Minimap:CreateTexture(nil,"Overlay")
			fmi:SetTexture("Interface\\AddOns\\shMinimap\\texture\\minimap")
			fmi:SetPoint("TOPLEFT", Minimap, "TOPLEFT", -3, 2)
			fmi:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", 3, 0)
			fmi:SetVertexColor(1, 1, 1)
		end
	end


---------------------------- track (blip) texture ------------------------------
	if bliptex == 1 then
		Minimap:SetBlipTexture"Interface\\AddOns\\shMinimap\\texture\\blipcharmed"		
	end

	for _, frame in pairs(frames) do
		frame:Hide()
	end
	frames = nil
end

---------------------------- calendar frame ------------------------------
-- hide it and let it show on right minimap click, tracking on middle click 
-- code from BasicMinimap by funkydude

	GameTimeFrame:Hide()
	GameTimeFrame:UnregisterAllEvents()
	GameTimeFrame.Show = kill

	MiniMapTracking:Hide()
	MiniMapTracking.Show = kill
	MiniMapTracking:UnregisterAllEvents()
	
	MinimapNorthTag:SetAlpha(1)
	--MiniMapMeetingStoneFrame:SetAlpha(0)	

	Minimap:SetScript("OnMouseUp", function(self, btn)
		if btn == "RightButton" then
			_G.GameTimeFrame:Click()
		elseif btn == "MiddleButton" then
			_G.ToggleDropDownMenu(1, nil, _G.MiniMapTrackingDropDown, self)
		else
			_G.Minimap_OnClick(self)
		end
	end)

-- calendar slashcmd
SlashCmdList["CALENDAR"] = function()
	ToggleCalendar()
end
SLASH_CALENDAR1 = "/cl"
SLASH_CALENDAR2 = "/calendar"
	
-- functions
Minimap:ClearAllPoints()
Minimap:SetPoint(anchorpoint, parent, relativepoint, xpos, ypos)
Minimap:SetScale(scale)

  ---------------------------------------------------
  -- Make it moveable 
  ---------------------------------------------------

  	Minimap:RegisterForDrag("LeftButton")
	Minimap:SetClampedToScreen(true)
	Minimap:SetMovable(true)
	Minimap:SetScript("OnDragStart", function(self) if IsAltKeyDown() and IsControlKeyDown() then self:StartMoving() end end)
	Minimap:SetScript("OnDragStop", function(self) if IsAltKeyDown() and IsControlKeyDown() then self:StopMovingOrSizing() end end)

Minimap:SetFrameStrata("MEDIUM")
addon:SetScript("OnEvent", event)
addon:RegisterEvent"PLAYER_LOGIN"