-- Title: Repulator
-- Version: 1.01
-- Revision Date: October 9, 2008
-- Author: Jeff "Taffu" Fancher
-- Notes: GPLv3 License
----------------------------------------------

local colorID = {
    [0] = { color = "|cffffffff", abbr = "(?)", },      -- Unknown
    [1] = { color = "|cffff0000", abbr = "(HTD)", },    -- Hated
    [2] = { color = "|cffff8c00", abbr = "(HST)", },    -- Hostile
    [3] = { color = "|cffff4500", abbr = "(UFD)", },    -- Unfriendly
    [4] = { color = "|cffffffff", abbr = "(NTL)", },    -- Neutral
    [5] = { color = "|cff008000", abbr = "(FRD)", },    -- Friendly
    [6] = { color = "|cff00ff7f", abbr = "(HON)", },    -- Honored
    [7] = { color = "|cffffff00", abbr = "(REV)", },    -- Revered
    [8] = { color = "|cffffd700", abbr = "(EXH)", },    -- Exhaulted
}

local RepUpdate, isWotLK

do
    local build = select(4, GetBuildInfo())
    if ( build >= 30000 ) then
        isWotLK = true
    else
        isWotLK = false
    end
end

if ( isWotLK ) then 
    RepUpdate = function()
        local numFactions = GetNumFactions();
        local factionIndex, factionRow, factionTitle, factionStanding, factionBar, factionButton, factionLeftLine, factionBottomLine, factionBackground, color, tooltipStanding;
        local name, description, standingID, barMin, barMax, barValue, barNeeded, atWarWith, canToggleAtWar, isHeader, isCollapsed, isWatched, isChild;
        local atWarIndicator, rightBarTexture;
        
        local previousBigTexture = ReputationFrameTopTreeTexture;	--In case we have a line going off the panel to the top
        previousBigTexture:Hide();
        local previousBigTexture2 = ReputationFrameTopTreeTexture2;
        previousBigTexture2:Hide();
        
        -- Update scroll frame
        if ( not FauxScrollFrame_Update(ReputationListScrollFrame, numFactions, NUM_FACTIONS_DISPLAYED, REPUTATIONFRAME_FACTIONHEIGHT ) ) then
            ReputationListScrollFrameScrollBar:SetValue(0);
        end
        local factionOffset = FauxScrollFrame_GetOffset(ReputationListScrollFrame);
        
        local gender = UnitSex("player");
        
        local i;
        
        local offScreenFudgeFactor = 5;
        local previousBigTextureRows = 0;
        local previousBigTextureRows2 = 0;
        for i=1, NUM_FACTIONS_DISPLAYED, 1 do
            factionIndex = factionOffset + i;
            factionRow = getglobal("ReputationBar"..i);
            factionBar = getglobal("ReputationBar"..i.."ReputationBar");
            factionTitle = getglobal("ReputationBar"..i.."FactionName");
            factionButton = getglobal("ReputationBar"..i.."ExpandOrCollapseButton");
            factionLeftLine = getglobal("ReputationBar"..i.."LeftLine");
            factionBottomLine = getglobal("ReputationBar"..i.."BottomLine");
            factionStanding = getglobal("ReputationBar"..i.."ReputationBarFactionStanding");
            factionBackground = getglobal("ReputationBar"..i.."Background");
            if ( factionIndex <= numFactions ) then
                name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild = GetFactionInfo(factionIndex);
                if ( not isHeader ) then name = format("%s %s%s|r", name, colorID[standingID].color, colorID[standingID].abbr) end
                factionTitle:SetText(name);
                if ( isCollapsed ) then
                    factionButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
                else
                    factionButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
                end
                factionRow.index = factionIndex;
                factionRow.isCollapsed = isCollapsed;
                local factionStandingtext = GetText("FACTION_STANDING_LABEL"..standingID, gender);
                
                --Normalize Values
                barMax = barMax - barMin;
                barValue = barValue - barMin;
                barNeeded = barMax - barValue;
                barMin = 0;
                
                if ( barMax == 1000 and barValue == 999 ) then
                    factionStanding:SetText();
                else
                    factionStanding:SetText(HIGHLIGHT_FONT_COLOR_CODE.." "..barValue.." / "..barMax.." "..FONT_COLOR_CODE_CLOSE);
                end
                if ( barMax == 1000 and barValue == 999 ) then
                    factionRow.tooltip = " ";
                    factionRow.standingText = " ";
                else
                    factionRow.tooltip = HIGHLIGHT_FONT_COLOR_CODE.." "..barNeeded.." "..FONT_COLOR_CODE_CLOSE;
                    factionRow.standingText = HIGHLIGHT_FONT_COLOR_CODE.." "..barValue.." / "..barMax.." "..FONT_COLOR_CODE_CLOSE;
                end
                factionBar:SetMinMaxValues(0, barMax);
                factionBar:SetValue(barValue);
                local color = FACTION_BAR_COLORS[standingID];
                factionBar:SetStatusBarColor(color.r, color.g, color.b);
                
                if ( isHeader and not isChild ) then
                    factionLeftLine:SetTexCoord(0, 0.25, 0, 2);
                    factionBottomLine:Hide();
                    factionLeftLine:Hide();
                    if ( previousBigTextureRows == 0 ) then
                        previousBigTexture:Hide();
                    end
                    previousBigTexture = factionBottomLine;
                    previousBigTextureRows = 0;
                elseif ( isHeader and isChild ) then
                    ReputationBar_DrawHorizontalLine(factionLeftLine, 11, factionButton);
                    if ( previousBigTexture2 and previousBigTextureRows2 == 0 ) then
                        previousBigTexture2:Hide();
                    end
                    factionBottomLine:Hide();
                    previousBigTexture2 = factionBottomLine;
                    previousBigTextureRows2 = 0;
                    previousBigTextureRows = previousBigTextureRows+1;
                    ReputationBar_DrawVerticalLine(previousBigTexture, previousBigTextureRows);
                elseif ( isChild ) then
                    ReputationBar_DrawHorizontalLine(factionLeftLine, 11, factionBackground);
                    factionBottomLine:Hide();
                    previousBigTextureRows = previousBigTextureRows+1;
                    previousBigTextureRows2 = previousBigTextureRows2+1;
                    ReputationBar_DrawVerticalLine(previousBigTexture2, previousBigTextureRows2);
                else
                    -- is immediately under a main category
                    ReputationBar_DrawHorizontalLine(factionLeftLine, 13, factionBackground);
                    factionBottomLine:Hide();
                    previousBigTextureRows = previousBigTextureRows+1;
                    ReputationBar_DrawVerticalLine(previousBigTexture, previousBigTextureRows);
                end
                
                ReputationFrame_SetRowType(factionRow, ((isChild and 1 or 0) + (isHeader and 2 or 0)), hasRep);
                
                factionRow:Show();
                
                -- Update details if this is the selected faction
                if ( atWarWith ) then
                    getglobal("ReputationBar"..i.."ReputationBarAtWarHighlight1"):Show();
                    getglobal("ReputationBar"..i.."ReputationBarAtWarHighlight2"):Show();
                else
                    getglobal("ReputationBar"..i.."ReputationBarAtWarHighlight1"):Hide();
                    getglobal("ReputationBar"..i.."ReputationBarAtWarHighlight2"):Hide();
                end
                if ( factionIndex == GetSelectedFaction() ) then
                    if ( ReputationDetailFrame:IsShown() ) then
                        ReputationDetailFactionName:SetText(name);
                        ReputationDetailFactionDescription:SetText(description);
                        if ( atWarWith ) then
                            ReputationDetailAtWarCheckBox:SetChecked(1);
                        else
                            ReputationDetailAtWarCheckBox:SetChecked(nil);
                        end
                        if ( canToggleAtWar and (not isHeader)) then
                            ReputationDetailAtWarCheckBox:Enable();
                            ReputationDetailAtWarCheckBoxText:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
                        else
                            ReputationDetailAtWarCheckBox:Disable();
                            ReputationDetailAtWarCheckBoxText:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
                        end
                        if ( not isHeader ) then
                            ReputationDetailInactiveCheckBox:Enable();
                            ReputationDetailInactiveCheckBoxText:SetTextColor(ReputationDetailInactiveCheckBoxText:GetFontObject():GetTextColor());
                        else
                            ReputationDetailInactiveCheckBox:Disable();
                            ReputationDetailInactiveCheckBoxText:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
                        end
                        if ( IsFactionInactive(factionIndex) ) then
                            ReputationDetailInactiveCheckBox:SetChecked(1);
                        else
                            ReputationDetailInactiveCheckBox:SetChecked(nil);
                        end
                        if ( isWatched ) then
                            ReputationDetailMainScreenCheckBox:SetChecked(1);
                        else
                            ReputationDetailMainScreenCheckBox:SetChecked(nil);
                        end
                        getglobal("ReputationBar"..i.."ReputationBarHighlight1"):Show();
                        getglobal("ReputationBar"..i.."ReputationBarHighlight2"):Show();
                    end
                else
                    getglobal("ReputationBar"..i.."ReputationBarHighlight1"):Hide();
                    getglobal("ReputationBar"..i.."ReputationBarHighlight2"):Hide();
                end
            else
                factionRow:Hide();
            end
        end
        if ( GetSelectedFaction() == 0 ) then
            ReputationDetailFrame:Hide();
        end
        
        local i = NUM_
        for i = (NUM_FACTIONS_DISPLAYED + factionOffset + 1), numFactions, 1 do
            local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild  = GetFactionInfo(i);
            if not name then break; end
    
            if ( isHeader and not isChild ) then
                break;
            elseif ( (isHeader and isChild) or not(isHeader or isChild) ) then
                ReputationBar_DrawVerticalLine(previousBigTexture, previousBigTextureRows+1);
                break;
            elseif ( isChild ) then
                ReputationBar_DrawVerticalLine(previousBigTexture2, previousBigTextureRows2+1);
                break;
            end
        end
    end
else
    RepUpdate = function()
        local numFactions = GetNumFactions();
        local factionIndex, factionName, factionCheck, factionStanding, factionBar, factionHeader, color, tooltipStanding;
        local name, description, standingID, barMin, barMax, barValue, barNeeded, atWarWith, canToggleAtWar, isHeader, isCollapsed, isWatched;
        local atWarIndicator, rightBarTexture;
    
        -- Update scroll frame
        if ( not FauxScrollFrame_Update(ReputationListScrollFrame, numFactions, NUM_FACTIONS_DISPLAYED, REPUTATIONFRAME_FACTIONHEIGHT ) ) then
            ReputationListScrollFrameScrollBar:SetValue(0);
        end
        local factionOffset = FauxScrollFrame_GetOffset(ReputationListScrollFrame);
    
        local gender = UnitSex("player");
        for i=1, NUM_FACTIONS_DISPLAYED, 1 do
            factionIndex = factionOffset + i;
            factionBar = getglobal("ReputationBar"..i);
            factionHeader = getglobal("ReputationHeader"..i);
            factionCheck = getglobal("ReputationBar"..i.."Check");
            if ( factionIndex <= numFactions ) then
                name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, isWatched = GetFactionInfo(factionIndex);
                if ( isHeader ) then
                    factionHeader:SetText("|cffffffff"..name.."|r");
                    if ( isCollapsed ) then
                        factionHeader:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
                    else
                        factionHeader:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
                    end
                    factionHeader.index = factionIndex;
                    factionHeader.isCollapsed = isCollapsed;
                    factionBar:Hide();
                    factionHeader:Show();
                    factionCheck:Hide();
                else
                    factionStanding = GetText("FACTION_STANDING_LABEL"..standingID, gender);
                    factionName = getglobal("ReputationBar"..i.."FactionName");
                    name = format("%s%s|r", colorID[standingID].color, name)
                    factionName:SetText(name)
                    getglobal("ReputationBar"..i.."FactionStanding"):SetText(factionStanding);
    
                    atWarIndicator = getglobal("ReputationBar"..i.."AtWarCheck");
                    rightBarTexture = getglobal("ReputationBar"..i.."ReputationBarRight");
    
                    if ( atWarWith ) then
                        atWarIndicator:Show();
                    else
                        atWarIndicator:Hide();
                    end
    
                    -- Normalize values
                    barMax = barMax - barMin;
                    barValue = barValue - barMin;
                    barNeeded = barMax - barValue;
                    barMin = 0;
    
                    factionBar.id = factionIndex;
                    factionBar.standingText = factionStanding;
                    if ( barMax == 1000 and barValue == 999 ) then
                        factionBar.tooltip = HIGHLIGHT_FONT_COLOR_CODE.." "..barValue.." / "..barMax..FONT_COLOR_CODE_CLOSE;
                    else
                        factionBar.tooltip = HIGHLIGHT_FONT_COLOR_CODE.." "..barValue.." / "..barMax.." ("..barNeeded..")"..FONT_COLOR_CODE_CLOSE;
                    end
                    factionBar:SetMinMaxValues(0, barMax);
                    factionBar:SetValue(barValue);
                    color = FACTION_BAR_COLORS[standingID];
                    factionBar:SetStatusBarColor(color.r, color.g, color.b);
                    factionBar:SetID(factionIndex);
                    factionBar:Show();
                    factionHeader:Hide();
    
                    -- Show a checkmark if this faction is being watched
                    if ( isWatched ) then
                        factionCheck:Show();
                        factionName:SetWidth(100);
                        factionCheck:SetPoint("LEFT", factionName, "LEFT", factionName:GetStringWidth(), 0);
                    else
                        factionCheck:Hide();
                        factionName:SetWidth(110);
                    end
    
                    -- Update details if this is the selected faction
                    if ( factionIndex == GetSelectedFaction() ) then
                        if ( ReputationDetailFrame:IsShown() ) then
                            ReputationDetailFactionName:SetText(name);
                            ReputationDetailFactionDescription:SetText(description);
                            if ( atWarWith ) then
                                ReputationDetailAtWarCheckBox:SetChecked(1);
                            else
                                ReputationDetailAtWarCheckBox:SetChecked(nil);
                            end
                            if ( canToggleAtWar ) then
                                ReputationDetailAtWarCheckBox:Enable();
                                ReputationDetailAtWarCheckBoxText:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
                            else
                                ReputationDetailAtWarCheckBox:Disable();
                                ReputationDetailAtWarCheckBoxText:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
    
                            end
                            if ( IsFactionInactive(factionIndex) ) then
                                ReputationDetailInactiveCheckBox:SetChecked(1);
                            else
                                ReputationDetailInactiveCheckBox:SetChecked(nil);
                            end
                            if ( isWatched ) then
                                ReputationDetailMainScreenCheckBox:SetChecked(1);
                            else
                                ReputationDetailMainScreenCheckBox:SetChecked(nil);
                            end
                        end
                        getglobal("ReputationBar"..i.."Highlight1"):Show();
                        getglobal("ReputationBar"..i.."Highlight2"):Show();
                    else
                        getglobal("ReputationBar"..i.."Highlight1"):Hide();
                        getglobal("ReputationBar"..i.."Highlight2"):Hide();
                    end
                end
            else
                factionHeader:Hide();
                factionBar:Hide();
            end
        end
        if ( GetSelectedFaction() == 0 ) then
            ReputationDetailFrame:Hide();
        end
    end
end

ReputationFrame_Update = RepUpdate